@@ -7,6 +7,7 @@ __all__ = [
 
 
 import re
+from dataclasses import replace
 from itertools import islice
 from pathlib import Path
 from typing import Dict, Iterable, Iterator, List, Mapping, Optional, Tuple, Union
@@ -23,6 +24,7 @@ from .fragment import Fragment
 
 # Patch markdown_it to allow arbitrary data urls
 # https://github.com/executablebooks/markdown-it-py/issues/128
+# TODO: Will soon be able to use custom validateLink
 normalize_url.GOOD_DATA_RE = re.compile(r"^data:")
 
 
@@ -59,6 +61,24 @@ class Extractor:
 
         return self.regex
 
+    def split(
+        self,
+        source: str,
+        directives: Mapping[str, Directive],
+    ) -> Iterator[Tuple[str, Optional[Fragment]]]:
+        """Split the given content into fragments."""
+        lines = source.splitlines(keepends=True)
+        previous_line = 0
+
+        for fragment in self.parse_fragments(source, directives):
+            if fragment.start_line > previous_line:
+                yield "".join(lines[previous_line : fragment.start_line]), None
+            yield "".join(lines[fragment.start_line : fragment.end_line]), fragment
+            previous_line = fragment.end_line
+
+        if previous_line < len(lines):
+            yield "".join(lines[previous_line:]), None
+
     def extract(
         self,
         source: str,
@@ -92,6 +112,8 @@ class Extractor:
 
     def create_fragment(
         self,
+        start_line: int,
+        end_line: int,
         match: "re.Match[str]",
         content: Optional[str] = None,
         url: Optional[str] = None,
@@ -100,6 +122,8 @@ class Extractor:
         """Helper for creating a fragment from a matched pattern."""
         directive, modifier, arguments = match.groups()
         return Fragment(
+            start_line=start_line,
+            end_line=end_line,
             directive=directive,
             modifier=modifier,
             arguments=arguments.split(),
@@ -121,7 +145,7 @@ class TextExtractor(Extractor):
         tokens = self.get_regex(directives).split(source + "\n")
 
         it = iter(tokens)
-        next(it)
+        newlines = next(it).count("\n")
 
         while True:
             try:
@@ -131,6 +155,8 @@ class TextExtractor(Extractor):
             else:
                 content = content.partition("\n")[-1]
                 yield Fragment(
+                    start_line=newlines,
+                    end_line=(newlines := newlines + content.count("\n") + 1),
                     directive=directive,
                     modifier=modifier,
                     arguments=arguments.split(),
@@ -178,11 +204,14 @@ class MarkdownExtractor(Extractor):
         tokens = self.parser.parse(source)  # type: ignore
         regex = self.get_regex(directives)
 
-        skip = 1
+        skip_to = 0
 
         for i, token in enumerate(tokens):
-            if skip > 1:
-                skip -= 1
+            if not token.map:
+                continue
+
+            current_line = token.map[0]
+            if skip_to > current_line:
                 continue
 
             #
@@ -194,7 +223,7 @@ class MarkdownExtractor(Extractor):
             #
             if (
                 (
-                    skip := self.match_tokens(
+                    skip_to := self.match_tokens(
                         tokens[i : i + 4],
                         "paragraph_open",
                         "inline",
@@ -207,7 +236,9 @@ class MarkdownExtractor(Extractor):
                 and self.match_tokens(inline.children, "code_inline")
                 and (match := regex.match(inline.children[0].content))
             ):
-                yield self.create_fragment(match, content=tokens[i + 3].content)
+                yield self.create_fragment(
+                    current_line, skip_to, match, content=tokens[i + 3].content
+                )
 
             #
             # `@directive args...`
@@ -216,7 +247,7 @@ class MarkdownExtractor(Extractor):
             #
             elif (
                 (
-                    skip := self.match_tokens(
+                    skip_to := self.match_tokens(
                         tokens[i : i + 6],
                         "paragraph_open",
                         "inline",
@@ -235,7 +266,9 @@ class MarkdownExtractor(Extractor):
                 and (link := image.children[0].attrGet("src"))
                 and (match := regex.match(inline.children[0].content))
             ):
-                yield self.create_link_fragment(match, link, external_files)
+                yield self.create_link_fragment(
+                    current_line, skip_to, match, link, external_files
+                )
 
             #
             # `@directive args...`
@@ -250,7 +283,7 @@ class MarkdownExtractor(Extractor):
             #
             elif (
                 (
-                    skip := self.match_tokens(
+                    skip_to := self.match_tokens(
                         tokens[i : i + 6],
                         "paragraph_open",
                         "inline",
@@ -267,7 +300,9 @@ class MarkdownExtractor(Extractor):
                 and tokens[i + 5].content == "</details>\n"
                 and (match := regex.match(inline.children[0].content))
             ):
-                yield self.create_fragment(match, content=tokens[i + 4].content)
+                yield self.create_fragment(
+                    current_line, skip_to, match, content=tokens[i + 4].content
+                )
 
             #
             # `@directive args...`
@@ -280,7 +315,7 @@ class MarkdownExtractor(Extractor):
             #
             elif (
                 (
-                    skip := self.match_tokens(
+                    skip_to := self.match_tokens(
                         tokens[i : i + 8],
                         "paragraph_open",
                         "inline",
@@ -303,14 +338,16 @@ class MarkdownExtractor(Extractor):
                 and (link := image.children[0].attrGet("src"))
                 and (match := regex.match(inline.children[0].content))
             ):
-                yield self.create_link_fragment(match, link, external_files)
+                yield self.create_link_fragment(
+                    current_line, skip_to, match, link, external_files
+                )
 
             #
             # [`@directive args...`](path/to/content)
             #
             elif (
                 (
-                    skip := self.match_tokens(
+                    skip_to := self.match_tokens(
                         tokens[i : i + 3],
                         "paragraph_open",
                         "inline",
@@ -328,7 +365,9 @@ class MarkdownExtractor(Extractor):
                 and (link := inline.children[0].attrGet("href"))
                 and (match := regex.match(inline.children[1].content))
             ):
-                yield self.create_link_fragment(match, link, external_files)
+                yield self.create_link_fragment(
+                    current_line, skip_to, match, link, external_files
+                )
 
             #
             # <!-- @directive args... -->
@@ -339,7 +378,7 @@ class MarkdownExtractor(Extractor):
             #
             elif (
                 (
-                    skip := self.match_tokens(
+                    skip_to := self.match_tokens(
                         tokens[i : i + 2],
                         "html_block",
                         ["fence", "code_block"],
@@ -348,7 +387,9 @@ class MarkdownExtractor(Extractor):
                 and (comment := self.html_comment_regex.match(token.content))
                 and (match := regex.match(comment.group(1)))
             ):
-                yield self.create_fragment(match, content=tokens[i + 1].content)
+                yield self.create_fragment(
+                    current_line, skip_to, match, content=tokens[i + 1].content
+                )
 
             #
             # <!-- @directive args... -->
@@ -357,7 +398,7 @@ class MarkdownExtractor(Extractor):
             #
             elif (
                 (
-                    skip := self.match_tokens(
+                    skip_to := self.match_tokens(
                         tokens[i : i + 4],
                         "html_block",
                         "paragraph_open",
@@ -372,14 +413,16 @@ class MarkdownExtractor(Extractor):
                 and (link := image.children[0].attrGet("src"))
                 and (match := regex.match(comment.group(1)))
             ):
-                yield self.create_link_fragment(match, link, external_files)
+                yield self.create_link_fragment(
+                    current_line, skip_to, match, link, external_files
+                )
 
             #
             # `@directive args...`
             #
             elif (
                 (
-                    skip := self.match_tokens(
+                    skip_to := self.match_tokens(
                         tokens[i : i + 3],
                         "paragraph_open",
                         "inline",
@@ -391,7 +434,7 @@ class MarkdownExtractor(Extractor):
                 and self.match_tokens(inline.children, "code_inline")
                 and (match := regex.match(inline.children[0].content))
             ):
-                yield self.create_fragment(match)
+                yield self.create_fragment(current_line, skip_to, match)
 
             #
             # <!-- @directive args... -->
@@ -401,7 +444,7 @@ class MarkdownExtractor(Extractor):
                 and (comment := self.html_comment_regex.match(token.content))
                 and (match := regex.match(comment.group(1)))
             ):
-                yield self.create_fragment(match)
+                yield self.create_fragment(current_line, skip_to, match)
 
             #
             # ```
@@ -411,10 +454,15 @@ class MarkdownExtractor(Extractor):
             # ```
             #
             elif token.type in ["fence", "code_block"]:
-                yield from self.embedded_extractor.parse_fragments(
+                for fragment in self.embedded_extractor.parse_fragments(
                     token.content,
                     directives,
-                )
+                ):
+                    yield replace(
+                        fragment,
+                        start_line=fragment.start_line + current_line,
+                        end_line=(skip_to := fragment.end_line + current_line),
+                    )
 
     def match_tokens(
         self,
@@ -433,17 +481,19 @@ class MarkdownExtractor(Extractor):
                 )
                 for token, token_type in zip(tokens, token_types)
             )
-            and len(tokens)
+            and next((token.map[-1] for token in reversed(tokens) if token.map), 1)
         )
 
     def create_link_fragment(
         self,
+        start_line: int,
+        end_line: int,
         match: "re.Match[str]",
         link: str,
         external_files: Optional[FileSystemPath] = None,
     ) -> Fragment:
         """Helper for creating a fragment from a link."""
-        url = unquote(link)
+        url = unquote(link)  # TODO: Will soon be able to use custom normalizeLink
         path = None
 
         if urlparse(url).path == url:
@@ -451,4 +501,4 @@ class MarkdownExtractor(Extractor):
                 path = Path(external_files, url).resolve()
             url = None
 
-        return self.create_fragment(match, url=url, path=path)
+        return self.create_fragment(start_line, end_line, match, url=url, path=path)
