@@ -4,28 +4,32 @@ __all__ = [
 ]
 
 
-from typing import Iterable, Optional
+import subprocess
+from typing import Iterable, List, Optional, cast
 
 from beet import Context
+from beet.core.utils import JsonDict
 from beet.toolchain.context import Plugin
 
 from .document import Document
 
 
 def beet_default(ctx: Context):
-    config = ctx.meta.get("lectern", {})
+    config = ctx.meta.get("lectern", cast(JsonDict, {}))
 
-    load = config.get("load", [])
+    load = config.get("load", ())
     snapshot = config.get("snapshot")
     external_files = config.get("external_files")
+    scripts = config.get("scripts", ())
 
-    ctx.require(lectern(load, snapshot, external_files))
+    ctx.require(lectern(load, snapshot, external_files, scripts))
 
 
 def lectern(
     load: Iterable[str] = (),
     snapshot: Optional[str] = None,
     external_files: Optional[str] = None,
+    scripts: Iterable[List[str]] = (),
 ) -> Plugin:
     """Return a plugin that handles markdown files with lectern."""
 
@@ -36,6 +40,10 @@ def lectern(
             for path in ctx.directory.glob(pattern):
                 document.load(path)
 
+        for arguments in scripts:
+            stdout = subprocess.check_output(arguments, cwd=ctx.directory).decode()
+            document.add_text(stdout)
+
         yield
 
         if snapshot:
